#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from tricircleclient.tests.unit import utils
from tricircleclient.v1 import pods_cli


class _TestPodCommand(utils.TestCommand):

    def setUp(self):
        super(_TestPodCommand, self).setUp()
        self.pod_manager = self.app.client_manager.multiregion_networking.pod


class TestCreatePod(_TestPodCommand, utils.TestCommandWithoutOptions):

    def setUp(self):
        super(TestCreatePod, self).setUp()
        self.cmd = pods_cli.CreatePod(self.app, None)

    def test_create_all_options(self):
        _pod = utils.FakePod.create_single_pod()
        arglist = [
            '--region-name', _pod['pod']['region_name'],
            '--availability-zone', _pod['pod']['az_name'],
            '--pod-availability-zone', _pod['pod']['pod_az_name'],
            '--data-center', _pod['pod']['dc_name']
        ]
        verifylist = [
            ('region_name', _pod['pod']['region_name']),
            ('availability_zone', _pod['pod']['az_name']),
            ('pod_availability_zone', _pod['pod']['pod_az_name']),
            ('data_center', _pod['pod']['dc_name']),
        ]
        self.pod_manager.create = mock.Mock(return_value=_pod)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = (self.cmd.take_action(parsed_args))

        self.assertEqual(sorted(_pod['pod'].keys()), sorted(columns))
        self.assertEqual(sorted(_pod['pod'].values()), sorted(data))

    def test_create_local_region_with_availability_zone(self):
        keys = ["dc_name", "pod_az_name"]
        _pod = utils.FakePod.create_single_pod({key: " " for key in keys})
        arglist = [
            '--region-name', _pod['pod']['region_name'],
            '--availability-zone', _pod['pod']['az_name']
        ]
        verifylist = [
            ('region_name', _pod['pod']['region_name']),
            ('availability_zone', _pod['pod']['az_name']),
        ]
        self.pod_manager.create = mock.Mock(return_value=_pod)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = (self.cmd.take_action(parsed_args))

        self.assertEqual(sorted(_pod['pod'].keys()), sorted(columns))
        self.assertEqual(sorted(_pod['pod'].values()), sorted(data))

    def test_create_central_region(self):
        keys = ["dc_name", "pod_az_name", "az_name"]
        _pod = utils.FakePod.create_single_pod({key: " " for key in keys})
        arglist = [
            '--region-name', _pod['pod']['region_name']
        ]
        verifylist = [
            ('region_name', _pod['pod']['region_name']),
        ]
        self.pod_manager.create = mock.Mock(return_value=_pod)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = (self.cmd.take_action(parsed_args))

        self.assertEqual(sorted(_pod['pod'].keys()), sorted(columns))
        self.assertEqual(sorted(_pod['pod'].values()), sorted(data))


class TestShowPod(_TestPodCommand, utils.TestCommandWithoutOptions):

    def setUp(self):
        super(TestShowPod, self).setUp()
        self.cmd = pods_cli.ShowPod(self.app, None)

    def test_show_valid_pod(self):
        _pod = utils.FakePod.create_single_pod()
        arglist = [
            _pod['pod']['pod_id'],
            ]
        verifylist = [
            ("pod", _pod['pod']['pod_id']),
            ]
        self.pod_manager.get = mock.Mock(return_value=_pod)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = (self.cmd.take_action(parsed_args))

        self.assertEqual(sorted(_pod['pod'].keys()), sorted(columns))
        self.assertEqual(sorted(_pod['pod'].values()), sorted(data))

    def test_show_valid_pod_with_empty_fields(self):
        keys = ["dc_name", "pod_az_name", "az_name"]
        _pod = utils.FakePod.create_single_pod({key: " " for key in keys})
        arglist = [
            _pod['pod']['pod_id'],
            ]
        verifylist = [
            ("pod", _pod['pod']['pod_id']),
            ]
        self.pod_manager.get = mock.Mock(return_value=_pod)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = (self.cmd.take_action(parsed_args))

        self.assertEqual(sorted(_pod['pod'].keys()), sorted(columns))
        self.assertEqual(sorted(_pod['pod'].values()), sorted(data))


class TestListPod(_TestPodCommand):

    columns = [
        'ID',
        'Region Name',
    ]

    def setUp(self):
        super(TestListPod, self).setUp()
        self.cmd = pods_cli.ListPods(self.app, None)

    def test_list(self):
        _pods = utils.FakePod.create_multiple_pods()
        for pod in _pods:
            pod.pop('dc_name')
            pod.pop('pod_az_name')
            pod.pop('az_name')
        self.pod_manager.list = mock.Mock(
            return_value={'pods': _pods})
        parsed_args = self.check_parser(self.cmd)
        columns, data = (self.cmd.take_action(parsed_args))

        self.assertEqual(self.columns, sorted(columns))
        self.assertEqual(len(_pods), len(data))


class TestDeletePod(_TestPodCommand, utils.TestCommandWithoutOptions):

    def setUp(self):
        super(TestDeletePod, self).setUp()
        self.cmd = pods_cli.DeletePod(self.app, None)

    def test_delete_pod(self):
        arglist = [utils.FakePod.create_single_pod()['pod']['pod_id']]
        verifylist = [
            ('pod', arglist),
            ]
        self.pod_manager.delete = mock.Mock(
            return_value=None)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)
        self.assertIsNone(result)

    def test_delete_multiple_pod(self):
        arglist = [pod['pod_id'] for pod in
                   utils.FakePod.create_multiple_pods()]
        verifylist = [
            ('pod', arglist),
            ]
        self.pod_manager.delete = mock.Mock(return_value=None)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)
        self.assertIsNone(result)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


def prepare_column_headers(columns, remap=None):
    remap = remap if remap else {}
    new_columns = []
    for c in columns:
        new_columns.append(remap[c].replace('_', ' '))

    return new_columns


def list2cols(cols, objs, headers=None):
    headers = headers if headers else cols
    return headers, [tuple([o[k] for k in cols])
                     for o in objs]



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_serialization import jsonutils

from tricircleclient.v1 import base


class PodManager(base.Manager):

    def list(self, search_opts=None):
        """Get a list of Pods."""
        return self._get(
            '/pods',
            headers={'Content-Type': "application/json"}).json()

    def create(self, pod):
        """Create a Pod."""
        return self._post(
            '/pods', headers={'Content-Type': "application/json"},
            data=jsonutils.dumps(pod)).json()

    def get(self, pod):
        """Get information about a Pod."""
        return self._get(
            '/pods/%s' % pod,
            headers={'Content-Type': "application/json"}).json()

    def delete(self, pod):
        """Delete a Pod."""
        self._delete('/pods/%s' % pod)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# asynchronous job types
JT_CONFIGURE_ROUTE = 'configure_route'
JT_ROUTER_SETUP = 'router_setup'
JT_PORT_DELETE = 'port_delete'
JT_SEG_RULE_SETUP = 'seg_rule_setup'
JT_NETWORK_UPDATE = 'update_network'
JT_SUBNET_UPDATE = 'subnet_update'
JT_SHADOW_PORT_SETUP = 'shadow_port_setup'
JT_TRUNK_SYNC = 'trunk_sync'
JT_SFC_SYNC = 'sfc_sync'
JT_RESOURCE_RECYCLE = 'resource_recycle'

# all resources needed to run the job. We specify it by
# {job_type: [resource_id1, resource_id2, ...]}.
job_resource_map = {
    JT_CONFIGURE_ROUTE: ["router_id"],
    JT_ROUTER_SETUP: ["pod_id", "router_id", "network_id"],
    JT_PORT_DELETE: ["pod_id", "port_id"],
    JT_SEG_RULE_SETUP: ["project_id"],
    JT_NETWORK_UPDATE: ["pod_id", "network_id"],
    JT_TRUNK_SYNC: ["pod_id", "trunk_id"],
    JT_SUBNET_UPDATE: ["pod_id", "subnet_id"],
    JT_SHADOW_PORT_SETUP: ["pod_id", "network_id"],
    JT_SFC_SYNC: ["pod_id", "portchain_id", "network_id"],
    JT_RESOURCE_RECYCLE: ["project_id"]
}

# job has many attributes, especially its resource attribute may vary with
# job type, To have unified column headers when show the job information,
# only the fields listed below will turn up in column headers. They are
# listed by alphabet order.
COLUMNS = ('id', 'project_id', 'status', 'timestamp', 'type')

# column headers about job that show in command line.
COLUMNS_REMAP = {'id': 'ID',
                 'project_id': 'Project',
                 'timestamp': 'Timestamp',
                 'status': 'Status',
                 'type': 'Type'}



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from tricircleclient import constants
from tricircleclient.tests.unit import utils
from tricircleclient.v1 import jobs_cli


class _TestJobCommand(utils.TestCommand):

    def setUp(self):
        super(_TestJobCommand, self).setUp()
        self.job_manager = self.app.client_manager.multiregion_networking.job


class TestCreateJob(_TestJobCommand, utils.TestCommandWithoutOptions):

    def setUp(self):
        super(TestCreateJob, self).setUp()
        self.cmd = jobs_cli.CreateJob(self.app, None)

    def test_create_all_options(self):
        _job = utils.FakeJob.create_single_job()
        arglist = [
            '--type', _job['job']['type'],
            '--project_id', _job['job']['project_id'],
            '--pod_id', _job['job']['resource']['pod_id'],
            '--router_id', _job['job']['resource']['router_id'],
            '--network_id', _job['job']['resource']['network_id'],
        ]
        verifylist = [
            ('type', _job['job']['type']),
            ('project_id', _job['job']['project_id']),
            ('pod_id', _job['job']['resource']['pod_id']),
            ('router_id', _job['job']['resource']['router_id']),
            ('network_id', _job['job']['resource']['network_id']),
        ]

        self.job_manager.create = mock.Mock(return_value=_job)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = (self.cmd.take_action(parsed_args))

        self.assertEqual(sorted(_job['job'].keys()), sorted(columns))
        self.assertEqual(sorted(_job['job'].values()), sorted(data))


class TestShowJob(_TestJobCommand, utils.TestCommandWithoutOptions):

    def setUp(self):
        super(TestShowJob, self).setUp()
        self.cmd = jobs_cli.ShowJob(self.app, None)

    def test_show_a_single_job(self):
        _job = utils.FakeJob.create_single_job()
        arglist = [
            _job['job']['id'],
            ]
        verifylist = [
            ('job', _job['job']['id']),
            ]
        self.job_manager.get = mock.Mock(return_value=_job)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = (self.cmd.take_action(parsed_args))

        self.assertEqual(sorted(_job['job'].keys()), sorted(columns))
        self.assertEqual(sorted(_job['job'].values()), sorted(data))


class TestListJob(_TestJobCommand):

    def setUp(self):
        super(TestListJob, self).setUp()
        self.cmd = jobs_cli.ListJobs(self.app, None)

    def test_list(self):
        _jobs = utils.FakeJob.create_multiple_jobs()
        self.job_manager.list = mock.Mock(return_value={'jobs': _jobs})
        parsed_args = self.check_parser(self.cmd)
        columns, data = (self.cmd.take_action(parsed_args))
        self.assertEqual(sorted(constants.COLUMNS_REMAP.values()),
                         sorted(columns))
        self.assertEqual(len(_jobs), len(data))
        self.assertEqual(
            sorted([tuple(o[k] for k in constants.COLUMNS) for o in _jobs]),
            sorted(data))

    def test_list_with_filters(self):
        _job = utils.FakeJob.create_single_job()
        _job = _job['job']

        # we filter the jobs by the following fields: project ID, type, status.
        # given values of _job, then only single item _job is retrieved.
        arglist = [
            '--project-id', _job['project_id'],
            '--type', _job['type'],
            '--status', _job['status'],
        ]
        verifylist = [
            ('project_id', _job['project_id']),
            ('type', _job['type']),
            ('status', _job['status'].lower()),
        ]

        self.job_manager.list = mock.Mock(return_value={'jobs':  [_job]})
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = (self.cmd.take_action(parsed_args))

        self.assertEqual(1, len(data))
        self.assertEqual(sorted(constants.COLUMNS_REMAP.values()),
                         sorted(columns))

        # lower case of job status
        arglist = [
            '--status', _job['status'].lower(),
        ]
        verifylist = [
            ('status', _job['status'].lower()),
        ]

        self.job_manager.list = mock.Mock(return_value={'jobs': [_job]})
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = (self.cmd.take_action(parsed_args))

        self.assertEqual(1, len(data))
        self.assertEqual(sorted(constants.COLUMNS_REMAP.values()),
                         sorted(columns))

    def test_invalid_job_status_filter(self):
        # unrecognizable job status filter
        arglist = [
            '--status', 'new_1',
        ]
        verifylist = []
        self.assertRaises(utils.ParserException, self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_list_with_pagination(self):
        number_of_jobs = 4
        limit = number_of_jobs - 2
        _jobs = utils.FakeJob.create_multiple_jobs(count=number_of_jobs)

        # test list operation with pagination
        arglist = [
            '--limit', str(limit),
        ]
        verifylist = [
            ('limit', limit),
        ]

        self.job_manager.list = mock.Mock(return_value={"jobs": _jobs[:limit]})
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = (self.cmd.take_action(parsed_args))

        self.assertEqual(limit, len(data))
        self.assertEqual(sorted(constants.COLUMNS_REMAP.values()),
                         sorted(columns))

        # test list operation with pagination and marker
        arglist = [
            '--limit', str(limit),
            '--marker', _jobs[0]['id'],
        ]
        verifylist = [
            ('limit', limit),
            ('marker', _jobs[0]['id']),
        ]

        self.job_manager.list = mock.Mock(
            return_value={"jobs": _jobs[1:limit+1]})
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = (self.cmd.take_action(parsed_args))

        self.assertEqual(limit, len(data))
        self.assertEqual(sorted(constants.COLUMNS_REMAP.values()),
                         sorted(columns))


class TestDeleteJob(_TestJobCommand, utils.TestCommandWithoutOptions):

    def setUp(self):
        super(TestDeleteJob, self).setUp()
        self.cmd = jobs_cli.DeleteJob(self.app, None)

    def test_delete_job(self):
        _job = utils.FakeJob.create_single_job()
        arglist = [
            _job['job']['id'],
            ]
        verifylist = [
            ('job', [_job['job']['id']]),
            ]
        self.job_manager.delete = mock.Mock(return_value=None)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.assertIsNone(result)


class TestRedoJob(_TestJobCommand, utils.TestCommandWithoutOptions):

    def setUp(self):
        super(TestRedoJob, self).setUp()
        self.cmd = jobs_cli.RedoJob(self.app, None)

    def test_redo_job(self):
        _job = utils.FakeJob.create_single_job()
        arglist = [
            _job['job']['id'],
        ]
        verifylist = [
            ('job', _job['job']['id']),
        ]
        self.job_manager.update = mock.Mock(return_value=None)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.assertIsNone(result)



#   Copyright 2017 Intel Corporation.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import sys

import mock


class FakeStdout(object):

    def __init__(self):
        self.content = []

    def write(self, text):
        self.content.append(text)

    def make_string(self):
        result = ''
        for line in self.content:
            result = result + line
        return result


class FakeLog(object):

    def __init__(self):
        self.messages = {}

    def debug(self, msg):
        self.messages['debug'] = msg

    def info(self, msg):
        self.messages['info'] = msg

    def warning(self, msg):
        self.messages['warning'] = msg

    def error(self, msg):
        self.messages['error'] = msg

    def critical(self, msg):
        self.messages['critical'] = msg


class FakeApp(object):

    def __init__(self, _stdout, _log):
        self.stdout = _stdout
        self.client_manager = None
        self.stdin = sys.stdin
        self.stdout = _stdout or sys.stdout
        self.stderr = sys.stderr
        self.log = _log


class FakeMultiregionNetworkV1Client(object):

    def __init__(self, **kwargs):
        self.pod = mock.Mock()
        self.routing = mock.Mock()
        self.job = mock.Mock()


class FakeClientManager(object):
    _api_version = {
        'tricircle': '1',
    }

    def __init__(self):
        self.multiregion_networking = FakeMultiregionNetworkV1Client()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from osc_lib.command import command
from oslo_log import log as logging
from six.moves.urllib import parse

from tricircleclient import utils


def _routing_from_args(parsed_args):
    data = {'top_id': parsed_args.top_id,
            'bottom_id': parsed_args.bottom_id,
            'pod_id': parsed_args.pod_id,
            'project_id': parsed_args.project_id,
            'resource_type': parsed_args.resource_type,
            'id': getattr(parsed_args, 'routing', None),
            }
    result = {}
    result.update((k, v) for k, v in data.items() if v)
    return {'routing': result}


def _add_pagination_argument(parser):
    parser.add_argument(
        '--limit',
        dest='limit', metavar="<num-routings>", type=int,
        help="Maximum number of routings to return",
        default=None)


def _add_marker_argument(parser):
    parser.add_argument(
        '--marker',
        dest='marker', metavar="<routing>", type=str,
        help="ID of last routing in previous page, routings after marker "
             "will be returned. Display all routings if not specified.",
        default=None)


def _add_filtering_arguments(parser):
    parser.add_argument(
        '--routing',
        dest='routing', metavar="<routing>", type=int,
        help="ID of a routing",
        default=None)
    parser.add_argument(
        '--top-id',
        dest='top_id', metavar="<top-id>", type=str,
        help="Resource id on Central Neutron",
        default=None)
    parser.add_argument(
        '--bottom-id',
        dest='bottom_id', metavar="<bottom-id>", type=str,
        help="Resource id on Local Neutron",
        default=None)
    parser.add_argument(
        '--pod-id',
        dest='pod_id', metavar="<pod-id>", type=str,
        help="ID of a pod",
        default=None)
    parser.add_argument(
        '--project-id',
        dest='project_id', metavar="<project-id>", type=str,
        help="ID of a project object in Keystone",
        default=None)
    parser.add_argument(
        '--resource-type',
        dest='resource_type', metavar="<resource-type>", type=str,
        choices=['network', 'subnet', 'port', 'router', 'security_group',
                 'trunk', 'port_pair', 'port_pair_group', 'flow_classifier',
                 'port_chain'],
        help="Available resource types",
        default=None)
    parser.add_argument(
        '--created-at',
        dest='created_at', metavar="<created-at>", type=str,
        help="Create time of the resource routing",
        default=None)
    parser.add_argument(
        '--updated-at',
        dest='updated_at', metavar="<updated-at>", type=str,
        help="Update time of the resource routing",
        default=None)


def _add_search_options(parsed_args):
    search_opts = {}
    for key in ('limit', 'marker', 'id', 'top_id', 'bottom_id', 'pod_id',
                'project_id', 'resource_type', 'created_at', 'updated_at'):
        value = getattr(parsed_args, key, None)
        if value is not None:
            search_opts[key] = value
    return search_opts


def _prepare_query_string(params):
    """Convert dict params to query string"""
    params = sorted(params.items(), key=lambda x: x[0])
    return '?%s' % parse.urlencode(params) if params else ''


class ListRoutings(command.Lister):
    """Lists Routings"""

    COLS = ('id', 'pod_id', 'resource_type', 'top_id')
    path = '/routings'

    log = logging.getLogger(__name__ + ".ListRoutings")

    def get_parser(self, prog_name):
        parser = super(ListRoutings, self).get_parser(prog_name)

        _add_pagination_argument(parser)
        _add_marker_argument(parser)
        _add_filtering_arguments(parser)

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.multiregion_networking

        # add pagination/marker/filter to list operation
        search_opts = _add_search_options(parsed_args)
        self.path += _prepare_query_string(search_opts)

        data = client.routing.list(self.path)
        remap = {'resource_type': 'Resource Type',
                 'pod_id': 'Pod',
                 'id': 'ID',
                 'top_id': 'Top',
                 }
        column_headers = utils.prepare_column_headers(self.COLS,
                                                      remap)
        return utils.list2cols(
            self.COLS, data['routings'], column_headers)


class CreateRouting(command.ShowOne):
    """Creates a Resource Routing"""

    log = logging.getLogger(__name__ + ".CreateRouting")

    def get_parser(self, prog_name):
        parser = super(CreateRouting, self).get_parser(prog_name)

        parser.add_argument(
            '--top-id',
            metavar="<top-id>",
            required=True,
            help="Resource id on Central Neutron",
        )
        parser.add_argument(
            '--bottom-id',
            metavar="<bottom-id>",
            required=True,
            help="Resource id on Local Neutron",
        )
        parser.add_argument(
            '--pod-id',
            metavar="<pod-id>",
            required=True,
            help="ID of a pod",
        )
        parser.add_argument(
            '--project-id',
            metavar="<project-id>",
            required=True,
            help="ID of a project object in Keystone",
        )
        parser.add_argument(
            '--resource-type',
            metavar="<resource-type>",
            choices=['network', 'subnet', 'port', 'router', 'security_group'],
            required=True,
            help="Available resource types",
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.multiregion_networking
        data = client.routing.create(_routing_from_args(parsed_args))

        return self.dict2columns(data['routing'])


class ShowRouting(command.ShowOne):
    """Display Routing Resource details."""

    log = logging.getLogger(__name__ + ".ShowRouting")

    def get_parser(self, prog_name):
        parser = super(ShowRouting, self).get_parser(prog_name)
        parser.add_argument(
            "routing",
            metavar="<routing>",
            help="ID of the routing resource to display",
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.multiregion_networking
        data = client.routing.get(parsed_args.routing)

        if 'routing' in data.keys():
            return self.dict2columns(data['routing'])


class DeleteRouting(command.Command):
    """Deletes Routing Resource."""

    log = logging.getLogger(__name__ + ".DeleteRouting")

    def get_parser(self, prog_name):
        parser = super(DeleteRouting, self).get_parser(prog_name)
        parser.add_argument(
            "routing",
            metavar="<routing>",
            nargs="+",
            help="ID(s) of the routing resource(s) to delete",
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.multiregion_networking
        for routing_id in parsed_args.routing:
            client.routing.delete(routing_id)


class UpdateRouting(command.Command):
    """Updates a Routing Resource."""

    log = logging.getLogger(__name__ + ".UpdateRouting")

    def get_parser(self, prog_name):
        parser = super(UpdateRouting, self).get_parser(prog_name)

        parser.add_argument(
            '--top-id',
            metavar="<top-id>",
            help="Resource id on Central Neutron",
        )
        parser.add_argument(
            '--bottom-id',
            metavar="<bottom-id>",
            help="Resource id on Local Neutron",
        )
        parser.add_argument(
            '--pod-id',
            metavar="<pod-id>",
            help="ID of a pod",
        )
        parser.add_argument(
            '--project-id',
            metavar="<project-id>",
            help="ID of a project object in Keystone",
        )
        parser.add_argument(
            '--resource-type',
            metavar="<resource-type>",
            choices=['network', 'subnet', 'port', 'router', 'security_group'],
            help="Available resource types",
        )
        parser.add_argument(
            "routing",
            metavar="<routing>",
            help="ID of the routing resource to update",
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.multiregion_networking
        client.routing.update(_routing_from_args(parsed_args))



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""OpenStackClient plugin for Multiregion Networking service."""

from osc_lib import utils


DEFAULT_MULTIREGION_NETWORKING_API_VERSION = '1'

# Required by the OSC plugin interface
API_NAME = 'multiregion_networking'
API_VERSION_OPTION = 'os_multiregion_networking_api_version'
API_VERSIONS = {
    '1': 'tricircleclient.v1.client.Client',
}


# Required by the OSC plugin interface
def make_client(instance):
    plugin_client = utils.get_client_class(
        API_NAME,
        instance._api_version[API_NAME],
        API_VERSIONS)

    instance.setup_auth()

    return plugin_client(
        session=instance.session,
        region_name=instance.region_name)


# Required by the OSC plugin interface
def build_option_parser(parser):
    """Hook to add global options."""
    parser.add_argument(
        "--os-multiregion-networking-api-version",
        metavar="<multiregion-networking-api-version>",
        default=utils.env(
            'OS_MULTIREGION_NETWORKING_API_VERSION',
            default=DEFAULT_MULTIREGION_NETWORKING_API_VERSION),
        help=("Multiregion Networking API version, default=" +
              DEFAULT_MULTIREGION_NETWORKING_API_VERSION +
              ' (Env: OS_MULTIREGION_NETWORKING_API_VERSION)'))
    parser.add_argument(
        "--os-multiregion-networking-url",
        default=utils.env(
            "OS_MULTIREGION_NETWORKING_URL"),
        help=("Data processing API URL, "
              "(Env: OS_MULTIREGION_NETWORKING_URL)"))
    return parser



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re
import six

from oslo_utils import encodeutils


@six.python_2_unicode_compatible
class ClientException(Exception):
    """The base exception class for all exceptions this library raises."""
    message = 'Unknown Error'

    def __init__(self, code=None, message=None, request_id=None,
                 url=None, method=None):
        self.code = code
        self.message = encodeutils.safe_decode(
            message or self.__class__.message)
        self.request_id = request_id
        self.url = url
        self.method = method

    def __str__(self):
        formatted_string = "%s" % self.message
        if self.code:
            formatted_string += " (HTTP %s)" % self.code
        if self.request_id:
            formatted_string += " (Request-ID: %s)" % self.request_id
        return formatted_string


class MutipleMeaningException(object):
    """An mixin for exception that can be enhanced by reading the details"""


class BadRequest(ClientException):
    """HTTP 400 - Bad request: you sent some malformed data."""
    http_status = 400
    message = "Bad request"


class Unauthorized(ClientException):
    """HTTP 401 - Unauthorized: bad credentials."""
    http_status = 401
    message = "Unauthorized"


class NotFound(ClientException):
    """HTTP 404 - Not found"""
    http_status = 404
    message = "Not found"


class PodNotFound(NotFound, MutipleMeaningException):
    message = "Pod not found"
    match = re.compile("Pod .* does not exist")


class Conflict(ClientException):
    """HTTP 409 - Conflict"""
    http_status = 409
    message = "Conflict"


class RecordAlreadyExists(Conflict, MutipleMeaningException):
    message = "Record already exists"
    match = re.compile("Record already exists")


class PodRegionDuplicated(Conflict, MutipleMeaningException):
    message = "Pod region name duplicated with the top region name"
    match = re.compile("Pod region name duplicated with the top region name")


class TopRegionAlreadyExists(Conflict, MutipleMeaningException):
    message = "Top region already exists"
    match = re.compile("Top region already exists")


class UnprocessableEntity(ClientException):
    """HTTP 422 - Unprocessable Entity"""
    http_status = 422
    message = "Unprocessable Entity"


class RegionRequiredForTopRegion(UnprocessableEntity, MutipleMeaningException):
    message = "Valid region_name is required for top region"
    match = re.compile("Valid region_name is required for top region")


class RegionRequiredForPod(UnprocessableEntity, MutipleMeaningException):
    message = "Valid region_name is required for pod"
    match = re.compile("Valid region_name is required for pod")


_error_classes = [BadRequest, Unauthorized, NotFound, Conflict]
_error_classes_enhanced = {
    NotFound: [PodNotFound, ],
    Conflict: [RecordAlreadyExists, PodRegionDuplicated,
               TopRegionAlreadyExists],
    UnprocessableEntity: [RegionRequiredForTopRegion, RegionRequiredForPod]
}
_code_map = dict(
    (c.http_status, (c, _error_classes_enhanced.get(c, [])))
    for c in _error_classes)


def from_response(response, method=None):
    """Return an instance of one of the ClientException on an requests response.

    Usage::

        resp, body = requests.request(...)
        if resp.status_code != 200:
            raise from_response(resp)

    """
    if response.status_code:
        cls, enhanced_classes = _code_map.get(response.status_code,
                                              (ClientException, []))

    req_id = response.headers.get("x-openstack-request-id")

    kwargs = {
        'code': response.status_code,
        'method': method,
        'url': response.url,
        'request_id': req_id,
    }

    if "retry-after" in response.headers:
        kwargs['retry_after'] = response.headers.get('retry-after')

    desc = response.text
    if desc:
        for enhanced_cls in enhanced_classes:
            if enhanced_cls.match.match(response.text):
                cls = enhanced_cls
                break
        kwargs['message'] = desc

    if not kwargs['message']:
        del kwargs['message']
    return cls(**kwargs)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from osc_lib.command import command
from oslo_log import log as logging
from six.moves.urllib import parse

from tricircleclient import constants
from tricircleclient import utils


def _job_from_args(parsed_args):
    # necessary parameters
    data = {'type': parsed_args.type,
            'project_id': parsed_args.project_id,
            }

    # optional parameters vary with job type
    resources = {}
    for id in constants.job_resource_map[data['type']]:
        resources[id] = getattr(parsed_args, id, None)
    data['resource'] = resources

    return {'job': data}


def _add_pagination_argument(parser):
    parser.add_argument(
        '--limit',
        dest='limit', metavar="<num-jobs>", type=int,
        help="Maximum number of jobs to return",
        default=None)


def _add_marker_argument(parser):
    parser.add_argument(
        '--marker',
        dest='marker', metavar="<job>", type=str,
        help="ID of last job in previous page, jobs after marker will be "
             "returned. Display all jobs if not specified.",
        default=None)


def _add_filtering_arguments(parser):
    # available filtering fields: project ID, type, status
    parser.add_argument(
        '--project-id',
        dest='project_id', metavar="<project-id>", type=str,
        help="ID of a project object in Keystone",
        default=None)
    parser.add_argument(
        '--type',
        dest='type', metavar="<type>", type=str,
        choices=constants.job_resource_map.keys(),
        help="Job type",
        default=None)
    parser.add_argument(
        '--status',
        dest='status', metavar="<status>", type=lambda str: str.lower(),
        choices=['new', 'running', 'success', 'fail'],
        help="Execution status of the job. It's case-insensitive",
        default=None)


def _add_search_options(parsed_args):
    search_opts = {}
    for key in ('limit', 'marker', 'project_id', 'type', 'status'):
        value = getattr(parsed_args, key, None)
        if value is not None:
            search_opts[key] = value
    return search_opts


def _prepare_query_string(params):
    """Convert dict params to query string"""
    params = sorted(params.items(), key=lambda x: x[0])
    return '?%s' % parse.urlencode(params) if params else ''


def expand_job_resource(job):
    # because job['resource'] is a dict value, so we should
    # expand its values and let them show as other fields in the
    # same level.
    for id in constants.job_resource_map[job['type']]:
        job[id] = job['resource'][id]
    job.pop('resource')
    return job


class ListJobs(command.Lister):
    """List Jobs"""
    log = logging.getLogger(__name__ + ".ListJobs")

    path = '/jobs'

    def get_parser(self, prog_name):
        parser = super(ListJobs, self).get_parser(prog_name)

        _add_pagination_argument(parser)
        _add_marker_argument(parser)
        _add_filtering_arguments(parser)

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.multiregion_networking

        # add pagination/marker/filter to list operation
        search_opts = _add_search_options(parsed_args)
        self.path += _prepare_query_string(search_opts)

        data = client.job.list(self.path)
        column_headers = utils.prepare_column_headers(constants.COLUMNS,
                                                      constants.COLUMNS_REMAP)
        return utils.list2cols(constants.COLUMNS, data['jobs'], column_headers)


class CreateJob(command.ShowOne):
    """Create a Job"""

    log = logging.getLogger(__name__ + ".CreateJob")

    def get_parser(self, prog_name):
        parser = super(CreateJob, self).get_parser(prog_name)

        # as resource is a compound attribute, so we expand its fields
        # and list them as optional parameters. If new resources are
        # provisioned, they should be added here.
        parser.add_argument(
            '--type',
            metavar="<type>",
            required=True,
            help="Job type",
        )
        parser.add_argument(
            '--project_id',
            metavar="<project-id>",
            required=True,
            help="ID of a project object in Keystone",
        )
        parser.add_argument(
            '--router_id',
            metavar="<router-id>",
            help="ID of a router",
        )
        parser.add_argument(
            '--network_id',
            metavar="<network-id>",
            help="ID of a network",
        )
        parser.add_argument(
            '--pod_id',
            metavar="<pod-id>",
            help="ID of a pod",
        )
        parser.add_argument(
            '--port_id',
            metavar="<port-id>",
            help="ID of a port",
        )
        parser.add_argument(
            '--trunk_id',
            metavar="<trunk-id>",
            help="ID of a trunk",
        )
        parser.add_argument(
            '--subnet_id',
            metavar="<subnet-id>",
            help="ID of a subnet",
        )
        parser.add_argument(
            '--portchain_id',
            metavar="<portchain-id>",
            help="ID of a port chain",
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.multiregion_networking
        data = client.job.create(_job_from_args(parsed_args))

        if 'job' in data.keys():
            return self.dict2columns(expand_job_resource(data['job']))


class ShowJob(command.ShowOne):
    """Display Job details."""
    log = logging.getLogger(__name__ + ".ShowJob")

    def get_parser(self, prog_name):
        parser = super(ShowJob, self).get_parser(prog_name)
        parser.add_argument(
            "job",
            metavar="<job>",
            help="ID of the job to display",
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.multiregion_networking
        data = client.job.get(parsed_args.job)

        if 'job' in data.keys():
            return self.dict2columns(expand_job_resource(data['job']))


class DeleteJob(command.Command):
    """Delete a Job."""

    log = logging.getLogger(__name__ + ".DeleteJob")

    def get_parser(self, prog_name):
        parser = super(DeleteJob, self).get_parser(prog_name)
        parser.add_argument(
            "job",
            metavar="<job>",
            nargs="+",
            help="ID(s) of the job(s) to delete",
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.multiregion_networking
        for job_id in parsed_args.job:
            client.job.delete(job_id)


class RedoJob(command.Command):
    """Redo a Job."""

    log = logging.getLogger(__name__ + ".RedoJob")

    def get_parser(self, prog_name):
        parser = super(RedoJob, self).get_parser(prog_name)

        parser.add_argument(
            'job',
            metavar="<job>",
            help="ID of the job to redo",
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.multiregion_networking
        client.job.update(parsed_args.job)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from tricircleclient.tests.unit import utils
from tricircleclient.v1 import routings_cli


class _TestRoutingCommand(utils.TestCommand):

    def setUp(self):
        super(_TestRoutingCommand, self).setUp()
        self.routing_manager = \
            self.app.client_manager.multiregion_networking.routing


class TestCreateRouting(_TestRoutingCommand, utils.TestCommandWithoutOptions):

    def setUp(self):
        super(TestCreateRouting, self).setUp()
        self.cmd = routings_cli.CreateRouting(self.app, None)

    def test_create_all_options(self):
        _routing = utils.FakeRouting.create_single_routing()
        arglist = [
            '--top-id', _routing['routing']['top_id'],
            '--bottom-id', _routing['routing']['bottom_id'],
            '--pod-id', _routing['routing']['pod_id'],
            '--project-id', _routing['routing']['project_id'],
            '--resource-type', _routing['routing']['resource_type'],
        ]
        verifylist = [
            ('top_id', _routing['routing']['top_id']),
            ('bottom_id', _routing['routing']['bottom_id']),
            ('pod_id', _routing['routing']['pod_id']),
            ('project_id', _routing['routing']['project_id']),
            ('resource_type', _routing['routing']['resource_type']),
        ]
        self.routing_manager.create = mock.Mock(return_value=_routing)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = (self.cmd.take_action(parsed_args))

        self.assertEqual(sorted(_routing['routing'].keys()), sorted(columns))
        self.assertEqual(sorted(_routing['routing'].values()), sorted(data))


class TestShowRouting(_TestRoutingCommand, utils.TestCommandWithoutOptions):

    def setUp(self):
        super(TestShowRouting, self).setUp()
        self.cmd = routings_cli.ShowRouting(self.app, None)

    def test_show_valid_routing(self):
        _routing = utils.FakeRouting.create_single_routing()
        arglist = [
            _routing['routing']['id'],
            ]
        verifylist = [
            ('routing', _routing['routing']['id']),
            ]
        self.routing_manager.get = mock.Mock(return_value=_routing)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = (self.cmd.take_action(parsed_args))

        self.assertEqual(sorted(_routing['routing'].keys()), sorted(columns))
        self.assertEqual(sorted(_routing['routing'].values()), sorted(data))


class TestListRouting(_TestRoutingCommand):

    columns = [
        'ID',
        'Pod',
        'Resource Type',
        'Top'
    ]

    def setUp(self):
        super(TestListRouting, self).setUp()
        self.cmd = routings_cli.ListRoutings(self.app, None)

    def test_list(self):
        _routings = utils.FakeRouting.create_multiple_routings()

        for routing in _routings:
            for key in ('bottom_id', 'project_id'):
                routing.pop(key)

        self.routing_manager.list = mock.Mock(
            return_value={'routings': _routings})
        parsed_args = self.check_parser(self.cmd)
        columns, data = (self.cmd.take_action(parsed_args))
        self.assertEqual(sorted(self.columns), sorted(columns))
        self.assertEqual(len(_routings), len(data))

    def test_list_with_filters(self):
        _routing = utils.FakeRouting.create_single_routing()
        _routing = _routing['routing']

        # we filter the routings by the following fields
        # given values of _routing, then only single item _routing
        # is retrieved.
        arglist = [
            '--top-id', _routing['top_id'],
            '--bottom-id', _routing['bottom_id'],
            '--pod-id', _routing['pod_id'],
            '--project-id', _routing['project_id'],
            '--resource-type', _routing['resource_type'],
            '--created-at', _routing['created_at'],
            '--updated-at', _routing['updated_at'],
        ]
        verifylist = [
            ('top_id', _routing['top_id']),
            ('bottom_id', _routing['bottom_id']),
            ('pod_id', _routing['pod_id']),
            ('project_id', _routing['project_id']),
            ('resource_type', _routing['resource_type']),
            ('created_at', _routing['created_at']),
            ('updated_at', _routing['updated_at']),
        ]
        for key in ('bottom_id', 'project_id'):
            _routing.pop(key)

        self.routing_manager.list = mock.Mock(
            return_value={'routings': [_routing]})
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = (self.cmd.take_action(parsed_args))

        self.assertEqual(self.columns, sorted(columns))
        self.assertEqual(1, len(data))

    def test_throw_exception(self):
        arglist = [
            '--resource-type', 'fake_resource_type',
        ]
        verifylist = []
        self.assertRaises(utils.ParserException, self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_list_with_pagination(self):
        _routings = utils.FakeRouting.create_multiple_routings(count=3)
        arglist = [
            '--limit', '2',
            '--marker', _routings[0]['id'],
        ]
        verifylist = [
            ('limit', 2),
            ('marker', _routings[0]['id']),
        ]
        for routing in _routings:
            for key in ('bottom_id', 'project_id'):
                routing.pop(key)

        self.routing_manager.list = mock.Mock(
            return_value={'routings': _routings[1:]})
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = (self.cmd.take_action(parsed_args))

        self.assertEqual(sorted(self.columns), sorted(columns))
        self.assertEqual(2, len(data))


class TestDeleteRouting(_TestRoutingCommand, utils.TestCommandWithoutOptions):

    def setUp(self):
        super(TestDeleteRouting, self).setUp()
        self.cmd = routings_cli.DeleteRouting(self.app, None)

    def test_delete_routing(self):
        _routings = utils.FakeRouting.create_multiple_routings(count=2)

        arglist = [
            _routings[0]['id'],
            _routings[1]['id'],
            ]
        verifylist = [
            ('routing', [_routings[0]['id'], _routings[1]['id']]),
            ]
        self.routing_manager.delete = mock.Mock(return_value=None)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.assertIsNone(result)

    def test_delete_multiple_routing(self):
        arglist = [routing['id'] for routing in
                   utils.FakeRouting.create_multiple_routings()]
        verifylist = [
            ('routing', arglist),
            ]
        self.routing_manager.delete = mock.Mock(return_value=None)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.assertIsNone(result)


class TestUpdateRouting(_TestRoutingCommand, utils.TestCommandWithoutOptions):

    def setUp(self):
        super(TestUpdateRouting, self).setUp()
        self.cmd = routings_cli.UpdateRouting(self.app, None)

    def test_update_all_options(self):
        _routing = utils.FakeRouting.create_single_routing()
        arglist = [
            '--top-id', _routing['routing']['top_id'],
            '--bottom-id', _routing['routing']['bottom_id'],
            '--project-id', _routing['routing']['project_id'],
            '--resource-type', _routing['routing']['resource_type'],
            _routing['routing']['pod_id'],
        ]
        verifylist = [
            ('top_id', _routing['routing']['top_id']),
            ('bottom_id', _routing['routing']['bottom_id']),
            ('project_id', _routing['routing']['project_id']),
            ('resource_type', _routing['routing']['resource_type']),
            ('routing', _routing['routing']['pod_id']),
        ]
        self.routing_manager.update = mock.Mock(return_value=_routing)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)



#   Copyright 2017 Intel Corporation.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import copy
from oslo_utils import timeutils
from oslo_utils import uuidutils
import testtools

from tricircleclient.tests.unit import fakes


class ParserException(Exception):
    pass


class TestCommand(testtools.TestCase):

    def setUp(self):
        super(TestCommand, self).setUp()
        # Build up a fake app
        self.fake_stdout = fakes.FakeStdout()
        self.fake_log = fakes.FakeLog()
        self.app = fakes.FakeApp(self.fake_stdout, self.fake_log)
        self.app.client_manager = fakes.FakeClientManager()

    def check_parser(self, cmd, args=[], verify_args=[]):
        cmd_parser = cmd.get_parser('check_parser')
        try:
            parsed_args = cmd_parser.parse_args(args)
        except SystemExit:
            raise ParserException("Argument parse failed")
        for attr, value in verify_args:
            if attr:
                self.assertIn(attr, parsed_args)
                self.assertEqual(value, getattr(parsed_args, attr))
        return parsed_args


class TestCommandWithoutOptions(object):

    def test_without_options(self):
        self.assertRaises(ParserException, self.check_parser, self.cmd)


class FakePod(object):
    """Fake one or more Pods."""

    @staticmethod
    def create_single_pod(opts=None):
        """Create a fake pod.

        :param opts:Dictionary of options to overwrite
        :return: A dictionary with dc_name, pod_name, pod_id,
        az_name, pod_az_name, region_name
        """
        opts = opts or {}
        # Set default options.
        fake_pod = {
            'pod': {
                'dc_name': 'datacenter',
                'pod_az_name': 'pod',
                'pod_id': uuidutils.generate_uuid(),
                'az_name': 'availability_zone',
                'region_name': 'central_region',
            }
        }

        # Overwrite default options
        fake_pod['pod'].update(opts)
        return copy.deepcopy(fake_pod)

    @staticmethod
    def create_multiple_pods(opts=None, count=2):
        """Create a list of fake pods.

        :param opts: A dictionary of options to create pod
        :param count: The number of pods to create
        :return: A list of dictionaries with dc_name, pod_name, pod_id,
        az_name, pod_az_name, region_name
        """
        return [FakePod.create_single_pod(opts)['pod']
                for i in range(0, count)]


class FakeRouting(object):
    """Fake one or more Routings."""

    @staticmethod
    def create_single_routing(opts=None):
        """Create a fake routing.

        :param opts: Dictionary of options to overwrite
        :return: A Dictionary with top_id, bottom_id, pod_id,
        project_id, resource_type, id
        """
        opts = opts or {}
        # Set default options.
        fake_routing = {
            'routing': {
                'top_id': uuidutils.generate_uuid(),
                'bottom_id': uuidutils.generate_uuid(),
                'pod_id': uuidutils.generate_uuid(),
                'project_id': uuidutils.generate_uuid(),
                'resource_type': 'network',
                'id': uuidutils.generate_uuid(),
                'created_at': str(timeutils.utcnow()),
                'updated_at': str(timeutils.utcnow()),
            }
        }

        # Overwrite default options
        fake_routing['routing'].update(opts)
        return copy.deepcopy(fake_routing)

    @staticmethod
    def create_multiple_routings(opts=None, count=2):
        """Create a list of fake routings.

        :param opts: A Dictionary of options to create routing
        :param count: The number of routings to create
        :return: A list of Dictionaries with top_id, bottom_id, pod_id,
        project_id, resource_type, id
        """
        return [FakeRouting.create_single_routing(opts)['routing']
                for i in range(0, count)]


class FakeJob(object):
    """Fake one or more Jobs."""

    @staticmethod
    def create_single_job(opts=None):
        """Create a fake job.

        :param opts: Dictionary of options to overwrite
        :return: A Dictionary with type, project_id. optional parameters are:
        router_id, network_id, pod_id, port_id, trunk_id, subnet_id,
        portchain_id. Different job type needs different resources, they
        are specified as optional parameters.
        """
        opts = opts or {}

        # Set default options.
        fake_job = {
            'job': {
                'id': uuidutils.generate_uuid(),
                'type': 'router_setup',
                'resource': {
                    'pod_id': uuidutils.generate_uuid(),
                    'router_id': uuidutils.generate_uuid(),
                    'network_id': uuidutils.generate_uuid(),
                },
                'project_id': uuidutils.generate_uuid(),
                'status': 'NEW',
                'timestamp': str(timeutils.utcnow()),
            }
        }

        # Overwrite default options
        fake_job['job'].update(opts)
        return fake_job

    @staticmethod
    def create_multiple_jobs(opts=None, count=2):
        """Create a list of fake jobs.

        :param opts: A Dictionary of options to create job
        :param count: The number of routings to create
        :return: A list of Dictionaries with type, project_id and some
        optional parameters. Optional paramaters are:
        router_id, network_id, pod_id, port_id, trunk_id, subnet_id,
        portchain_id. Different job type needs different resources, they
        are specified as optional parameters.
        """
        return [FakeJob.create_single_job(opts)['job']
                for i in range(0, count)]



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from osc_lib.command import command
from oslo_log import log as logging

from tricircleclient import utils


class ListPods(command.Lister):
    """Lists pods"""

    COLS = ('pod_id', 'region_name')

    log = logging.getLogger(__name__ + ".ListPods")

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.multiregion_networking
        data = client.pod.list()
        remap = {'pod_id': 'ID',
                 'region_name': 'Region Name',
                 'az_name': 'Availability Zone',
                 'dc_name': 'Data Center'}
        column_headers = utils.prepare_column_headers(self.COLS,
                                                      remap)

        return utils.list2cols(
            self.COLS, data['pods'], column_headers)


class CreatePod(command.ShowOne):
    """Creates pod"""

    log = logging.getLogger(__name__ + ".CreatePod")

    @staticmethod
    def _pod_from_args(parsed_args):
        result = {}
        result['region_name'] = parsed_args.region_name
        if parsed_args.availability_zone:
            result['az_name'] = parsed_args.availability_zone
        if parsed_args.pod_availability_zone:
            result['pod_az_name'] = parsed_args.pod_availability_zone
        if parsed_args.data_center:
            result['dc_name'] = parsed_args.data_center
        return {'pod': result}

    def get_parser(self, prog_name):
        parser = super(CreatePod, self).get_parser(prog_name)

        parser.add_argument(
            '--region-name',
            metavar="<region_name>",
            required=True,
            help="Region name registered in Keystone",
        )
        parser.add_argument(
            '--availability-zone',
            metavar="<az_name>",
            help="Name of the Availability Zone",
        )
        parser.add_argument(
            '--pod-availability-zone',
            metavar="<pod_az_name>",
            help="Name of the Availability Zone forwarded to local Neutron",
        )
        parser.add_argument(
            '--data-center',
            metavar="<dc_name>",
            help="Name of the Data Center",
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.multiregion_networking
        data = client.pod.create(self._pod_from_args(parsed_args))
        return self.dict2columns(data['pod'])


class ShowPod(command.ShowOne):
    """Display pod details."""

    log = logging.getLogger(__name__ + ".ShowPod")

    def get_parser(self, prog_name):
        parser = super(ShowPod, self).get_parser(prog_name)
        parser.add_argument(
            "pod",
            metavar="<pod>",
            help="Id of the pod to display",
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.multiregion_networking
        data = client.pod.get(parsed_args.pod)

        if 'pod' in data.keys():
            return self.dict2columns(data['pod'])


class DeletePod(command.Command):
    """Deletes Pod."""

    log = logging.getLogger(__name__ + ".DeletePod")

    def get_parser(self, prog_name):
        parser = super(DeletePod, self).get_parser(prog_name)
        parser.add_argument(
            "pod",
            metavar="<pod>",
            nargs="+",
            help="ID(s) of the pod(s) to delete",
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        client = self.app.client_manager.multiregion_networking
        for pod_id in parsed_args.pod:
            client.pod.delete(pod_id)
